@@ -1,16 +1,17 @@
 # (c) 2011-2012 Dennis Kaarsemaker <dennis@kaarsemaker.net>
 # see COPYING for license details
 
+import os
+import random
+import re
 import socket
 import cStringIO as StringIO
-import re
 import sys
+import warnings
 try:
     import xml.etree.cElementTree as etree
 except ImportError:
     import cElementTree as etree
-import uuid
-import warnings
 
 # Which protocol to use
 ILO_RAW  = 1
@@ -42,7 +43,7 @@ class Ilo(object):
 
     XML_HEADER = '<?xml version="1.0"?>\r\n'
     HTTP_HEADER = "POST /ribcl HTTP/1.1\r\nHost: localhost\r\nContent-Length: %d\r\nConnection: Close%s\r\n\r\n"
-    HTTP_UPLOAD_HEADER = "POST /cgi-bin/uploadRibclFiles HTTP/1.1\r\nHost: localhost\r\nContent-Length: %d\r\nContent-Type: multipart/form-data; boundary=%s\r\n\r\n"
+    HTTP_UPLOAD_HEADER = "POST /cgi-bin/uploadRibclFiles HTTP/1.1\r\nHost: localhost\r\nConnection: Close\r\nContent-Length: %d\r\nContent-Type: multipart/form-data; boundary=%s\r\n\r\n"
     BLOCK_SIZE = 4096
 
     def __init__(self, hostname, login, password, timeout=60, port=443):
@@ -53,15 +54,20 @@ class Ilo(object):
         self.debug    = 0
         self.protocol = None
         self.port     = port
+        self.cookie   = None
 
     def __str__(self):
         return "iLO interface of %s" % self.hostname
 
     def _debug(self, level, message):
         if self.debug >= level:
-            print >>sys.stderr, re.sub(r'PASSWORD=".*"','PASSWORD="********"', message)
+            if message.startswith('\r'):
+                sys.stderr.write(re.sub(r'PASSWORD=".*"','PASSWORD="********"', message))
+                sys.stderr.flush()
+            else:
+                print >>sys.stderr, re.sub(r'PASSWORD=".*"','PASSWORD="********"', message)
 
-    def _request(self, xml):
+    def _request(self, xml, progress=None):
         """Given an ElementTree.Element, serialize it and do the request.
            Returns an ElementTree.Element containing the response"""
 
@@ -74,7 +80,7 @@ class Ilo(object):
         else:
             xml = etree.tostring(xml)
 
-        header, data =  self._communicate(xml, self.protocol)
+        header, data =  self._communicate(xml, self.protocol, progress=progress)
 
         # This thing usually contains multiple XML messages
         messages = []
@@ -108,11 +114,60 @@ class Ilo(object):
         else:
             self.protocol = ILO_RAW
 
-    def _upload_file(self, filename):
-        boundary = 'hpilo-firmware-upload-%s' % uuid.uuid4()
+    def _upload_file(self, filename, progress):
+        firmware = open(filename, 'rb').read()
+        boundary = '------hpiLO3t' + str(random.randint(100000,1000000)) + 'z'
+        while boundary in firmware:
+            boundary = '------hpiLO3t' + str(random.randint(100000,1000000)) + 'z'
+        parts = [
+            """--%s\r\nContent-Disposition: form-data; name="fileType"\r\n\r\n""" % boundary,
+            """\r\n--%s\r\nContent-Disposition: form-data; name="fwimgfile"; filename="%s"\r\nContent-Type: application/octet-stream\r\n\r\n""" % (boundary, filename),
+            firmware,
+            """\r\n--%s--\r\n""" % boundary
+        ]
+        total_bytes = sum([len(x) for x in parts])
+        sock = self._get_socket()
+
+        self._debug(2, self.HTTP_UPLOAD_HEADER % (total_bytes, boundary))
+        sock.write(self.HTTP_UPLOAD_HEADER % (total_bytes, boundary))
+        for part in parts:
+            if len(part) < 2048:
+                self._debug(2, part)
+                sock.write(part)
+            else:
+                sent = 0
+                pkglen = 2048
+                fwlen = len(part)
+                while sent < fwlen:
+                    written = sock.write(part[sent:sent+pkglen])
+                    sent += written
+                    if callable(progress):
+                        progress("\r\033[KSent %d/%d bytes (%d%%)" % (sent, fwlen, 100.0*sent/fwlen))
+                    else:
+                        self._debug(2, "\r\033[KSent %d/%d bytes (%d%%)" % (sent, fwlen, 100.0*sent/fwlen))
+                if callable(progress):
+                    progress("")
+                else:
+                    self._debug(2, "")
 
 
-    def _communicate(self, xml, protocol, extra_header=''):
+        data = ''
+        try:
+            while True:
+                d = sock.read()
+                data += d
+                if not d:
+                    break
+        except socket.sslerror, e: # Connection closed
+            if not data:
+                raise IloError("Communication with %s:%d failed: %s" % (self.hostname, self.port, str(e)))
+
+        self._debug(1, "Received %d bytes" % len(data))
+        self._debug(2, data)
+        self.cookie = re.search('Set-Cookie: *(.*)', data).groups(1)
+        self._debug(2, "Cookie: %s" % self.cookie)
+
+    def _get_socket(self):
         """Set up an https connection and do an HTTP/raw socket request"""
         sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         sock.settimeout(self.timeout)
@@ -124,12 +179,17 @@ class Ilo(object):
         except socket.error, e:
             raise IloError("Error connecting to %s:%d: %s" % (self.hostname, self.port, str(e)))
         try:
-            sock = socket.ssl(sock)
+            return socket.ssl(sock)
         except socket.sslerror, e:
             raise IloError("Cannot establish ssl session with %s:%d: %s" % (self.hostname, self.port, e.message))
 
+    def _communicate(self, xml, protocol, progress=None):
+        sock = self._get_socket()
         msglen = msglen_ = len(self.XML_HEADER + xml)
         if protocol == ILO_HTTP:
+            extra_header = ''
+            if self.cookie:
+                extra_header = "\r\nCookie: %s" % self.cookie
             http_header = self.HTTP_HEADER % (msglen, extra_header)
             msglen += len(http_header)
         self._debug(1, "Sending XML request, %d bytes" % msglen)
@@ -142,7 +202,27 @@ class Ilo(object):
 
         # XML header and data need to arrive in 2 distinct packets
         sock.write(self.XML_HEADER)
-        sock.write(xml)
+        if '$EMBED' in xml:
+            pre, name, post = re.compile(r'(.*)\$EMBED:(.*)\$(.*)', re.DOTALL).match(xml).groups()
+            sock.write(pre)
+            sent = 0
+            fwlen = os.path.getsize(name)
+            fw = open(name, 'rb').read()
+            pkglen = 2048
+            while sent < fwlen:
+                written = sock.write(fw[sent:sent+pkglen])
+                sent += written
+                if callable(progress):
+                    progress("\r\033[KSent %d/%d bytes (%d%%)" % (sent, fwlen, 100.0*sent/fwlen))
+                else:
+                    self._debug(2, "\r\033[KSent %d/%d bytes (%d%%)" % (sent, fwlen, 100.0*sent/fwlen))
+            if callable(progress):
+                progress("")
+            else:
+                self._debug(2, "")
+            sock.write(post.strip())
+        else:
+            sock.write(xml)
 
         # And grab the data
         data = ''
@@ -152,7 +232,17 @@ class Ilo(object):
                 data += d
                 if not d:
                     break
-        except socket.sslerror: # Connection closed
+                if callable(progress) and d.strip().endswith('</RIBCL>'):
+                    d = d[d.find('<?xml'):]
+                    while '<?xml' in d:
+                        end = d.find('<?xml', 5)
+                        if end == -1:
+                            progress(self._parse_message(d, include_inform=True))
+                            break
+                        else:
+                            progress(self._parse_message(d[:end], include_inform=True))
+                            d = d[end:]
+        except socket.sslerror, e: # Connection closed
             if not data:
                 raise IloError("Communication with %s:%d failed: %s" % (self.hostname, self.port, str(e)))
 
@@ -193,7 +283,7 @@ class Ilo(object):
         element = etree.SubElement(login, element, **attrs)
         return root, element
 
-    def _parse_message(self, data):
+    def _parse_message(self, data, include_inform=False):
         """Parse iLO responses into Element instances and remove useless messages"""
         # Bug in some ilo versions causes malformed XML
         if '<RIBCL VERSION="2.22"/>' in data:
@@ -205,9 +295,12 @@ class Ilo(object):
         message = etree.fromstring(data)
         if message.tag == 'RIBCL':
             for child in message:
-                # INFORM messages are useless
                 if child.tag == 'INFORM':
-                    pass
+                    if include_inform:
+                        # Filter useless message:
+                        if 'should be updated' in child.text:
+                            return None
+                        return child.text
                 # RESPONE with status 0 also adds no value
                 elif child.tag == 'RESPONSE' and int(child.get('STATUS'), 16) == 0:
                     if child.get('MESSAGE') != 'No error':
@@ -233,8 +326,12 @@ class Ilo(object):
         """Returns a dict with tag names of all child elements as keys and the
            VALUE attributes as values"""
         retval = {}
+        keys = [elt.tag.lower() for elt in element]
+        if len(keys) != 1 and len(set(keys)) == 1:
+            # Can't return a dict
+            retval = []
         for elt in element:
-            key, val, unit = elt.tag.lower(), elt.get('VALUE', None), elt.get('UNIT', None)
+            key, val, unit = elt.tag.lower(), elt.get('VALUE', elt.get('value', None)), elt.get('UNIT', None)
             if val is None:
                 # HP is not best friends with consistency. Sometimes there are
                 # attributes, sometimes child tags and sometimes text nodes. Oh
@@ -249,9 +346,15 @@ class Ilo(object):
             val = self._coerce(val)
 
             if unit:
-                retval[key] = (val, unit)
+                if isinstance(retval, list):
+                    retval.append((val,unit))
+                else:
+                    retval[key] = (val, unit)
             else:
-                retval[key] = val
+                if isinstance(retval, list):
+                    retval.append(val)
+                else:
+                    retval[key] = val
         return retval
 
     def _element_to_dict(self, element):
@@ -322,9 +425,10 @@ class Ilo(object):
         header, message = self._request(root)
         if message is None:
             return None
-        # Code path below is untested. Error out for now.
-        raise IloError("You've reached unknown territories, please report a bug")
         message = message.find(returntag or tagname)
+        if message.text.strip():
+            return message.text.strip()
+        raise IloError("You've reached unknown territories, please report a bug")
         if list(message):
             return self._element_children_to_dict(message)
         else:
@@ -336,7 +440,7 @@ class Ilo(object):
         return self._control_tag('RIB_INFO', 'LICENSE', elements=[license])
 
     def add_user(self, user_login, user_name, password, admin_priv=False,
-            remote_cons_prive=True, reset_server_priv=False,
+            remote_cons_priv=True, reset_server_priv=False,
             virtual_media_priv=False, config_ilo_priv=True):
         """Add a new user to the iLO interface with the specified name,
            password and permissions. Permission attributes should be boolean
@@ -350,6 +454,15 @@ class Ilo(object):
         return self._control_tag('USER_INFO', 'ADD_USER', elements=elements,
                 attrib={'USER_LOGIN': user_login, 'USER_NAME': user_name, 'PASSWORD': password})
 
+    @untested
+    def cert_fqdn(self, use_fqdn):
+        use_fqdn = str({True: 'Yes', False: 'No'}.get(use_fqdn, use_fqdn))
+        return self._control_tag('RIB_INFO', 'CERT_FQDN', attrib={'VALUE': use_fqdn})
+
+    def certificate_signing_request(self):
+        """Get a certificate signing request from the iLO"""
+        return self._control_tag('RIB_INFO', 'CERTIFICATE_SIGNING_REQUEST')
+
     def clear_ilo_event_log(self):
         """Clears the iLO event log"""
         return self._control_tag('RIB_INFO', 'CLEAR_EVENTLOG')
@@ -362,6 +475,17 @@ class Ilo(object):
         """Clears the server power on time"""
         return self._control_tag('SERVER_INFO', 'CLEAR_SERVER_POWER_ON_TIME')
 
+    def computer_lock_config(self, computer_lock=None, computer_lock_key=None):
+        """Configure the computer lock settings"""
+        if computer_lock_key:
+            computer_lock = "custom"
+        if not computer_lock:
+            raise IloError("A value must be specified for computer_lock")
+        elements = [etree.Element('COMPUTER_LOCK', VALUE=computer_lock)]
+        if computer_lock_key:
+            elements.append(etree.Element('COMPUTER_LOCK_KEY', VALUE=computer_lock_key))
+        return self._control_tag('RIB_INFO', 'COMPUTER_LOCK_CONFIG', elements=elements)
+
     def delete_user(self, user_login):
         """Delete the specified user from the ilo"""
         return self._control_tag('USER_INFO', 'DELETE_USER', attrib={'USER_LOGIN': user_login})
@@ -374,6 +498,10 @@ class Ilo(object):
         """Eject the virtual media attached to the specified device"""
         return self._control_tag('RIB_INFO', 'EJECT_VIRTUAL_MEDIA', attrib={"DEVICE": device.upper()})
 
+    def factory_defaults(self):
+        """Reset the iLO to factory default settings"""
+        return self._control_tag('RIB_INFO', 'FACTORY_DEFAULTS')
+
     def get_all_users(self):
         """Get a list of all loginnames"""
         data = self._info_tag2('USER_INFO', 'GET_ALL_USERS', key='value')
@@ -383,6 +511,10 @@ class Ilo(object):
         """Get basic and authorization info of all users"""
         return self._info_tag2('USER_INFO', 'GET_ALL_USER_INFO', key='user_login')
 
+    def get_cert_subject_info(self):
+        """Get ssl certificate subject information"""
+        return self._info_tag('RIB_INFO', 'GET_CERT_SUBJECT_INFO')
+
     def get_dir_config(self):
         """Get directory authentication configuration"""
         return self._info_tag('DIR_INFO', 'GET_DIR_CONFIG')
@@ -448,6 +580,10 @@ class Ilo(object):
             records.append(record_)
         return records
 
+    def get_host_power_saver_status(self):
+        """Get the configuration of the ProLiant power regulator"""
+        return self._info_tag('SERVER_INFO', 'GET_HOST_POWER_SAVER_STATUS', 'GET_HOST_POWER_SAVER')
+
     def get_host_power_status(self):
         """Whether the server is powered on or not"""
         data = self._info_tag('SERVER_INFO', 'GET_HOST_POWER_STATUS', 'GET_HOST_POWER')
@@ -478,15 +614,13 @@ class Ilo(object):
         """Get information about the OA of the enclosing chassis"""
         return self._info_tag('BLADESYSTEM_INFO', 'GET_OA_INFO')
 
-    @untested
     def get_one_time_boot(self):
         """Get the one time boot state of the host"""
-        return self._info_tag('SERVER_INFO', 'GET_ONE_TIME_BOOT')
+        return self._info_tag('SERVER_INFO', 'GET_ONE_TIME_BOOT', 'ONE_TIME_BOOT')
 
-    @untested
     def get_persistent_boot(self):
         """Get the boot order of the host"""
-        return self._info_tag('SERVER_INFO', 'GET_PERSISTENT_BOOT')
+        return self._info_tag('SERVER_INFO', 'GET_PERSISTENT_BOOT', 'PERSISTENT_BOOT')
 
     def get_power_cap(self):
         """Get the power cap setting"""
@@ -560,6 +694,14 @@ class Ilo(object):
         """Get the status of virtual media devices. Valid devices are FLOPPY and CDROM"""
         return self._info_tag('RIB_INFO', 'GET_VM_STATUS', attrib={'DEVICE': device})
 
+    def hotkey_config(self, ctrl_t=None, ctrl_u=None, ctrl_v=None, ctrl_w=None,
+                      ctrl_x=None, ctrl_y=None):
+        """Change a set of shortcuts"""
+        vars = locals()
+        del vars['self']
+        elements = [etree.Element(x.upper(), VALUE=vars[x]) for x in vars if vars[x] is not None]
+        return self._control_tag('RIB_INFO', 'HOTKEY_CONFIG', elements=elements)
+
     # Not sure how this would work, and I have no relevant hardware
     #def insert_virtual_floppy(self, device, image_location):
     #    """Insert a virtual floppy"""
@@ -594,39 +736,52 @@ class Ilo(object):
 
     def mod_global_settings(self, session_timeout=None, f8_prompt_enabled=None,
             f8_login_required=None, lock_configuration=None,
-            http_port=None, https_port=None,
-            ssh_port=None, ssh_status=None, virtual_media_port=None,
-            min_password=None, enfoce_aes=None,
-            authentication_failure_logging=None, rbsu_post_ip=None):
-        """Modify iLO global settings, only values that are specified will be
-           changed. Remote console settings can be changed with
-   :func:`mod_remote_console_settings` as the function signature for
-           this function is ridiculous enough already"""
-        vars = dict(locals())
-        del vars['self']
-        elements = [etree.Element(x.upper(), VALUE=str({True: 'Yes', False: 'No'}.get(vars[x], vars[x])))
-                    for x in vars if vars[x] is not None]
-        return self._control_tag('RIB_INFO', 'MOD_GLOBAL_SETTINGS', elements=elements)
-
-    def mod_remote_console_settings(self, remote_console_port=None,
-            remote_console_encryption=None, remote_keyboard_model=None,
+            http_port=None, https_port=None, ssh_port=None, ssh_status=None,
+            vmedia_disable=None, virtual_media_port=None, remote_console_port=None,
+            min_password=None, enfoce_aes=None, authentication_failure_logging=None,
+            rbsu_post_ip=None, remote_console_encryption=None, remote_keyboard_model=None,
             terminal_services_port=None, high_performance_mouse=None,
             shared_console_enable=None, shared_console_port=None,
             remote_console_acquire=None):
-        """Modify remote console settings, only values that are specified will be changed"""
-        vars = dict(locals())
-        del vars['self']
-        elements = [etree.Element(x.upper(), VALUE=str({True: 'Yes', False: 'No'}.get(vars[x], vars[x])))
-                    for x in vars if vars[x] is not None]
-        return self._control_tag('RIB_INFO', 'MOD_GLOBAL_SETTINGS', elements=elements)
+        """Modify iLO global settings, only values that are specified will be changed."""
         vars = dict(locals())
         del vars['self']
         elements = [etree.Element(x.upper(), VALUE=str({True: 'Yes', False: 'No'}.get(vars[x], vars[x])))
                     for x in vars if vars[x] is not None]
         return self._control_tag('RIB_INFO', 'MOD_GLOBAL_SETTINGS', elements=elements)
 
+    def mod_network_settings(self, enable_nic=None, reg_ddns_server=None,
+            ping_gateway=None, dhcp_domain_name=None, speed_autoselect=None,
+            nic_speed=None, full_duplex=None, dhcp_enable=None,
+            ip_address=None, subnet_mask=None, gateway_ip_address=None,
+            dns_name=None, domain_name=None, dhcp_gateway=None,
+            dhcp_dns_server=None, dhcp_wins_server=None, dhcp_static_route=None,
+            reg_wins_server=None, prim_dns_server=None, sec_dns_server=None,
+            ter_dns_server=None, prim_wins_server=None, sec_wins_server=None,
+            static_route_1=None, static_route_2=None, static_route_3=None,
+            dhcp_sntp_settings=None, sntp_server1=None, sntp_server2=None,
+            timezone=None, enclosure_ip_enable=None, web_agent_ip_address=None,
+            shared_network_port=None, vlan_enabled=None, vlan_id=None,
+            shared_network_port_vlan=None, shared_network_port_vlan_id=None):
+        """Configure the network settings for the iLO card"""
+        vars = dict(locals())
+        del vars['self']
+        elements = [etree.Element(x.upper(), VALUE=str({True: 'Yes', False: 'No'}.get(vars[x], vars[x])))
+                    for x in vars if vars[x] is not None]
+        return self._control_tag('RIB_INFO', 'MOD_NETWORK_SETTINGS', elements=elements)
+
+    def mod_snmp_im_settings(self, web_agent_ip_address=None, snmp_address_1=None,
+            snmp_address_2=None, snmp_address_3=None, os_traps=None,
+            snmp_passthrough_status=None, rib_traps=None, cim_security_mask=None):
+        """Configure the SNMP and Insight Manager integration settings"""
+        vars = dict(locals())
+        del vars['self']
+        elements = [etree.Element(x.upper(), VALUE=str({True: 'Yes', False: 'No'}.get(vars[x], vars[x])))
+                    for x in vars if vars[x] is not None]
+        return self._control_tag('RIB_INFO', 'MOD_SNMP_IM_SETTINGS', elements=elements)
+
     def mod_user(self, user_login, user_name=None, password=None,
-            admin_priv=None, remote_cons_prive=None, reset_server_priv=None,
+            admin_priv=None, remote_cons_priv=None, reset_server_priv=None,
             virtual_media_priv=None, config_ilo_priv=None):
         """Set attributes for a user, only specified arguments will be changed.
            All arguments except user_name and password should be boolean"""
@@ -643,6 +798,22 @@ class Ilo(object):
 
         return self._control_tag('USER_INFO', 'MOD_USER', attrib={'USER_LOGIN': user_login}, elements=elements)
 
+    def press_pwr_btn(self):
+        """Press the power button"""
+        return self._control_tag('SERVER_INFO', 'PRESS_PWR_BTN')
+
+    def hold_pwr_btn(self):
+        """Press and hold the power button"""
+        return self._control_tag('SERVER_INFO', 'HOLD_PWR_BTN')
+
+    def cold_boot_server(self):
+        """Force a cold boot of the server"""
+        return self._control_tag('SERVER_INFO', 'COLD_BOOT_SERVER')
+
+    def warm_boot_server(self):
+        """Force a warm boot of the server"""
+        return self._control_tag('SERVER_INFO', 'WARM_BOOT_SERVER')
+
     def reset_rib(self):
         """Reset the iLO/RILOE board"""
         return self._control_tag('RIB_INFO', 'RESET_RIB')
@@ -651,11 +822,28 @@ class Ilo(object):
         """Power cycle the server"""
         return self._control_tag('SERVER_INFO', 'RESET_SERVER')
 
+    def set_language(self, lang_id):
+        """Set the default language"""
+        return self._control_tag('RIB_INFO', 'SET_LANGUAGE', attrib={'LANG_ID': lang_id})
+
     def set_host_power(self, host_power=True):
         """Turn host power on or off"""
         power = ['No', 'Yes'][bool(host_power)]
         return self._control_tag('SERVER_INFO', 'SET_HOST_POWER', attrib={'HOST_POWER': power})
 
+    def set_host_power_saver(self, host_power_saver):
+        """Set the configuration of the ProLiant power regulator"""
+        return self._control_tag('SERVER_INFO', 'SET_HOST_POWER_SAVER', attrib={'HOST_POWER_SAVER': str(host_power_saver)})
+
+    def set_one_time_boot(self, device):
+        """Set one time boot device"""
+        return self._control_tag('SERVER_INFO', 'SET_ONE_TIME_BOOT', attrib={'VALUE': device.upper()})
+
+    def set_persistent_boot(self, devices):
+        """Set persistent boot order, devices should be comma-separated"""
+        elements = [etree.Element('DEVICE', VALUE=x.upper()) for x in devices.split(',')]
+        return self._control_tag('SERVER_INFO', 'SET_PERSISTENT_BOOT', elements=elements)
+
     @untested
     def set_power_cap(self, power_cap):
         """Set the power cap feature to a specific value"""
@@ -703,18 +891,34 @@ class Ilo(object):
             raise ValueError("uid should be Yes or No")
         return self._control_tag('SERVER_INFO', 'UID_CONTROL', attrib={"UID": uid.title()})
 
-    def update_rib_firmware(self, filename):
+    def update_rib_firmware(self, filename, progress=None):
         """Upload new RIB firmware"""
-        size = os.path.getsize(filename)
         if not self.protocol:
             self._detect_protocol()
 
-        if self.protocol == ILO_RAW:
-            # We need to violate XML here. Dirty.
-            pass
+        if progress:
+            def progress_(data):
+                if data is None:
+                    return
+                elif isinstance(data, basestring):
+                    if '%' in data:
+                        data = '\r\033[K' + data
+                    progress(data)
+                else:
+                    raise RuntimeError("Unknown progress message")
         else:
-            cookie = self._upload_file(filename)
+            progress_ = None
 
+        fwlen = os.path.getsize(filename)
+        root, inner = self._root_element('RIB_INFO', MODE='write')
+        etree.SubElement(inner, 'TPM_ENABLED', VALUE='Yes')
+        inner = etree.SubElement(inner, 'UPDATE_RIB_FIRMWARE', IMAGE_LOCATION=filename, IMAGE_LENGTH=str(fwlen))
+        if self.protocol == ILO_RAW:
+            inner.tail = '$EMBED:%s$' % filename
+            return self._request(root, progress_)[1]
+        else:
+            self._upload_file(filename, progress_)
+            return self._request(root, progress_)[1]
 
 
 ##############################################################################################
@@ -740,11 +944,6 @@ class Ilo(object):
         """FIXME: I have no relevant hardware. Please report sample output"""
         return self._raw(('SERVER_INFO', {'MODE': 'READ'}), ('GET_HOST_POWER_REG_INFO', {}))
 
-    @untested
-    def get_host_power_saver_status(self):
-        """FIXME: I have no relevant hardware. Please report sample output"""
-        return self._raw(('SERVER_INFO', {'MODE': 'READ'}), ('GET_HOST_POWER_SAVER_STATUS', {}))
-
     @untested
     def get_topology(self):
         """FIXME: I have no relevant hardware. Please report sample output"""
